==> eslint-plugin-enforce-react-memo-1.3.0.tgz: package/src/rules/memoizeComponent.ts <==
import {
    RuleFixer,
    RuleContext,
} from '@typescript-eslint/utils/dist/ts-eslint';
import { AST_NODE_TYPES, TSESTree } from '@typescript-eslint/utils';
import createRule from '../util/createRule';

const memoizeComponent = createRule({
    name: 'memoize-component',
    meta: {
        docs: {
            description: 'React components should be memoized',
            recommended: 'warn',
        },
        messages: {
            general: 'Memoize this component',
        },
        type: 'problem',
        schema: [
            {
                type: 'object',
                properties: {},
                additionalProperties: false,
            },
        ],
        fixable: 'code',
    },
    defaultOptions: [],
    create(context) {
        function* fix(
            context: RuleContext<'general' | 'import-memo', never[]>,
            fixer: RuleFixer,
            node: TSESTree.Node
        ) {
            yield fixer.insertTextBefore(node, 'memo(');

            yield fixer.insertTextAfter(node, ')');
        }

        const verifyVariableDeclaration = (
            node: TSESTree.VariableDeclaration
        ) => {
            const declaration = node.declarations[0];

            if (declaration.type === AST_NODE_TYPES.VariableDeclarator) {
                const init = declaration.init;
                if (!init) {
                    // Not a component
                    return;
                }

                const name =
                    declaration.id.type === AST_NODE_TYPES.Identifier
                        ? declaration.id.name
                        : undefined;

                if (
                    !name ||
                    // Shallow checking for component declaration
                    name.charAt(0) !== name.charAt(0).toUpperCase()
                ) {
                    return;
                }

                const defaultExportNode = (
                    (node.parent as any).body as TSESTree.Node[]
                )?.find?.(item => {
                    if (item.type === AST_NODE_TYPES.ExportDefaultDeclaration) {
                        const line = context.getSourceCode().getLines()[
                            item.loc.start.line - 1
                        ];

                        if (line.includes(name)) {
                            return true;
                        }
                    }

                    return false;
                }) as TSESTree.ExportDefaultDeclaration | undefined;

                if (init.type === AST_NODE_TYPES.ArrowFunctionExpression) {
                    if (!defaultExportNode) {
                        context.report({
                            node,
                            messageId: 'general',
                            *fix(fixer) {
                                yield* fix(context, fixer, init);
                            },
                        });

                        return;
                    }

                    const isDefaultExportMemoized = (() => {
                        const exportDeclaration = (
                            defaultExportNode as TSESTree.ExportDefaultDeclaration
                        ).declaration;

                        if (
                            exportDeclaration &&
                            exportDeclaration.type ===
                                AST_NODE_TYPES.CallExpression
                        ) {
                            const callee = exportDeclaration.callee;
                            if (
                                callee.type === AST_NODE_TYPES.Identifier &&
                                callee.name === 'memo'
                            ) {
                                return true;
                            }
                        }
                    })();

                    if (isDefaultExportMemoized) {
                        return;
                    }

                    context.report({
                        node: defaultExportNode,
                        messageId: 'general',
                        *fix(fixer) {
                            yield* fix(
                                context,
                                fixer,
                                defaultExportNode.declaration
                            );
                        },
                    });

                    return;
                }

                if (init.type === AST_NODE_TYPES.CallExpression) {
                    const callee = init.callee;
                    const args = init.arguments;
                    const calleeChain = args
                        .filter(
                            item =>
                                item.type === AST_NODE_TYPES.CallExpression &&
                                item.callee.type === AST_NODE_TYPES.Identifier
                        )
                        .map(
                            (item: TSESTree.CallExpressionArgument) =>
                                (
                                    (item as TSESTree.CallExpression)
                                        .callee as TSESTree.Identifier
                                ).name
                        );

                    if (callee.type === AST_NODE_TYPES.Identifier) {
                        calleeChain.push(callee.name);
                    }

                    if (!calleeChain.includes('memo')) {
                        context.report({
                            node,
                            messageId: 'general',
                            *fix(fixer) {
                                yield* fix(context, fixer, init);
                            },
                        });

                        return;
                    }
                }
            }
        };

        const verifyExportNamedDeclaration = (
            node: TSESTree.ExportNamedDeclaration
        ) => {
            if (node.declaration?.type === AST_NODE_TYPES.VariableDeclaration) {
                verifyVariableDeclaration(node.declaration);
            }
        };

        const findRootNode = (
            currentNode: TSESTree.Node
        ):
            | TSESTree.VariableDeclaration
            | TSESTree.ExportNamedDeclaration
            | undefined => {
            if (
                [
                    AST_NODE_TYPES.VariableDeclaration,
                    AST_NODE_TYPES.ExportNamedDeclaration,
                ].includes(currentNode.type)
            ) {
                return currentNode as
                    | TSESTree.VariableDeclaration
                    | TSESTree.ExportNamedDeclaration;
            }

            const parent = currentNode.parent;
            if (!parent) {
                return undefined;
            }

            return findRootNode(parent);
        };

        const processDeclaration = (
            declaration:
                | TSESTree.VariableDeclaration
                | TSESTree.ExportNamedDeclaration
        ) => {
            if (declaration.type === AST_NODE_TYPES.VariableDeclaration) {
                verifyVariableDeclaration(declaration);
                return;
            }

            if (declaration.type === AST_NODE_TYPES.ExportNamedDeclaration) {
                verifyExportNamedDeclaration(declaration);
            }
        };

        return {
            JSXElement(node) {
                if (node.parent?.type !== AST_NODE_TYPES.ReturnStatement) {
                    return;
                }

                const root = findRootNode(node);
                if (!root) {
                    return;
                }

                processDeclaration(root);
            },
            JSXFragment(node) {
                if (node.parent?.type !== AST_NODE_TYPES.ReturnStatement) {
                    return;
                }

                const root = findRootNode(node);
                if (!root) {
                    return;
                }

                processDeclaration(root);
            },
        };
    },
});

export default memoizeComponent;

==> eslint-plugin-enforce-react-memo-1.3.2.tgz: package/src/rules/memoizeComponent.ts <==
import {
    RuleFixer,
    RuleContext,
} from '@typescript-eslint/utils/dist/ts-eslint';
import { AST_NODE_TYPES, TSESTree } from '@typescript-eslint/utils';
import createRule from '../util/createRule';

const memoizeComponent = createRule({
    name: 'memoize-component',
    meta: {
        docs: {
            description: 'React components should be memoized',
            recommended: 'warn',
        },
        messages: {
            general: 'Memoize this component',
        },
        type: 'problem',
        schema: [
            {
                type: 'object',
                properties: {},
                additionalProperties: false,
            },
        ],
        fixable: 'code',
    },
    defaultOptions: [],
    create(context) {
        function* fix(
            context: RuleContext<'general' | 'import-memo', never[]>,
            fixer: RuleFixer,
            node: TSESTree.Node
        ) {
            yield fixer.insertTextBefore(node, 'memo(');

            yield fixer.insertTextAfter(node, ')');
        }

        const verifyVariableDeclaration = (
            node: TSESTree.VariableDeclaration
        ) => {
            const declaration = node.declarations[0];

            if (declaration.type === AST_NODE_TYPES.VariableDeclarator) {
                const init = declaration.init;
                if (!init) {
                    // Not a component
                    return;
                }

                const name =
                    declaration.id.type === AST_NODE_TYPES.Identifier
                        ? declaration.id.name
                        : undefined;

                if (
                    !name ||
                    // Shallow checking for component declaration
                    name.charAt(0) !== name.charAt(0).toUpperCase()
                ) {
                    return;
                }

                const defaultExportNode = (
                    (node.parent as any).body as TSESTree.Node[]
                )?.find?.(item => {
                    if (item.type === AST_NODE_TYPES.ExportDefaultDeclaration) {
                        const line = context.getSourceCode().getLines()[
                            item.loc.start.line - 1
                        ];

                        if (line.includes(name)) {
                            return true;
                        }
                    }

                    return false;
                }) as TSESTree.ExportDefaultDeclaration | undefined;

                const isDefaultExportMemoized =
                    !!defaultExportNode &&
                    (() => {
                        const exportDeclaration = (
                            defaultExportNode as TSESTree.ExportDefaultDeclaration
                        ).declaration;

                        if (
                            exportDeclaration &&
                            exportDeclaration.type ===
                                AST_NODE_TYPES.CallExpression
                        ) {
                            const callee = exportDeclaration.callee;
                            if (
                                callee.type === AST_NODE_TYPES.Identifier &&
                                callee.name === 'memo'
                            ) {
                                return true;
                            }
                        }
                    })();

                if (init.type === AST_NODE_TYPES.ArrowFunctionExpression) {
                    if (!defaultExportNode) {
                        context.report({
                            node,
                            messageId: 'general',
                            *fix(fixer) {
                                yield* fix(context, fixer, init);
                            },
                        });

                        return;
                    }

                    if (isDefaultExportMemoized) {
                        return;
                    }

                    context.report({
                        node: defaultExportNode,
                        messageId: 'general',
                        *fix(fixer) {
                            yield* fix(
                                context,
                                fixer,
                                defaultExportNode.declaration
                            );
                        },
                    });

                    return;
                }

                if (init.type === AST_NODE_TYPES.CallExpression) {
                    if (isDefaultExportMemoized) {
                        return;
                    }

                    const callee = init.callee;
                    const args = init.arguments;
                    const calleeChain = args
                        .filter(
                            item =>
                                item.type === AST_NODE_TYPES.CallExpression &&
                                item.callee.type === AST_NODE_TYPES.Identifier
                        )
                        .map(
                            (item: TSESTree.CallExpressionArgument) =>
                                (
                                    (item as TSESTree.CallExpression)
                                        .callee as TSESTree.Identifier
                                ).name
                        );

                    if (callee.type === AST_NODE_TYPES.Identifier) {
                        calleeChain.push(callee.name);
                    }

                    if (!calleeChain.includes('memo')) {
                        if (defaultExportNode) {
                            context.report({
                                node: defaultExportNode,
                                messageId: 'general',
                                *fix(fixer) {
                                    yield* fix(
                                        context,
                                        fixer,
                                        defaultExportNode.declaration
                                    );
                                },
                            });

                            return;
                        }

                        context.report({
                            node,
                            messageId: 'general',
                            *fix(fixer) {
                                yield* fix(context, fixer, init);
                            },
                        });

                        return;
                    }
                }
            }
        };

        const verifyExportNamedDeclaration = (
            node: TSESTree.ExportNamedDeclaration
        ) => {
            if (node.declaration?.type === AST_NODE_TYPES.VariableDeclaration) {
                verifyVariableDeclaration(node.declaration);
            }
        };

        const findRootNode = (
            currentNode: TSESTree.Node
        ):
            | TSESTree.VariableDeclaration
            | TSESTree.ExportNamedDeclaration
            | undefined => {
            if (
                [
                    AST_NODE_TYPES.VariableDeclaration,
                    AST_NODE_TYPES.ExportNamedDeclaration,
                ].includes(currentNode.type)
            ) {
                return currentNode as
                    | TSESTree.VariableDeclaration
                    | TSESTree.ExportNamedDeclaration;
            }

            const parent = currentNode.parent;
            if (!parent) {
                return undefined;
            }

            return findRootNode(parent);
        };

        const processDeclaration = (
            declaration:
                | TSESTree.VariableDeclaration
                | TSESTree.ExportNamedDeclaration
        ) => {
            if (declaration.type === AST_NODE_TYPES.VariableDeclaration) {
                verifyVariableDeclaration(declaration);
                return;
            }

            if (declaration.type === AST_NODE_TYPES.ExportNamedDeclaration) {
                verifyExportNamedDeclaration(declaration);
            }
        };

        return {
            JSXElement(node) {
                if (node.parent?.type !== AST_NODE_TYPES.ReturnStatement) {
                    return;
                }

                const root = findRootNode(node);
                if (!root) {
                    return;
                }

                processDeclaration(root);
            },
            JSXFragment(node) {
                if (node.parent?.type !== AST_NODE_TYPES.ReturnStatement) {
                    return;
                }

                const root = findRootNode(node);
                if (!root) {
                    return;
                }

                processDeclaration(root);
            },
        };
    },
});

export default memoizeComponent;
